/*
 * File:	wb_dialg.h
 * Purpose:	wxDialogBox and common dialog declarations
 * Author:	Julian Smart
 * Created:	1993
 * Updated:	
 * Copyright:	(c) 1993, AIAI, University of Edinburgh
 */

/* sccsid[] = "%W% %G%" */

#ifndef wxb_dialgh
#define wxb_dialgh

#include "common.h"
#include "wx_panel.h"

#ifdef IN_CPROTO
typedef       void    *wxbDialogBox ;
#else

// Dialog boxes
class wxbDialogBox: public wxPanel
{
 public:
  Bool modal;
  Bool is_show;

  wxbDialogBox(void);
  wxbDialogBox(wxFrame *frame, char *title, Bool modal = FALSE,
              int x = -1, int y = -1,
              int width = -1, int height = -1, int style = 0, char *name = "panel");
  ~wxbDialogBox();

  Bool Create(wxFrame *frame, char *title, Bool modal = FALSE,
              int x = -1, int y = -1,
              int width = -1, int height = -1, int style = 0, char *name = "panel");

  virtual void SetSize(int x, int y, int width, int height) = 0;
  virtual void SetClientSize(int width, int height) = 0;
  virtual void GetPosition(int *x, int *y) = 0;
  virtual void Show(Bool show) = 0;
  virtual void Iconize(Bool iconize) = 0;
  virtual Bool Iconized(void) = 0;
  // Set title
  virtual void SetTitle(char *title) = 0;
  virtual char *GetTitle(void) = 0;

  inline char *GetLabel(void) { return GetTitle(); }

  virtual void Fit(void) = 0;
  void Centre(int direction = wxBOTH);
};

// Handy dialog functions
char *wxGetTextFromUser(char *message, char *caption = "Input text",
                        char *default_value = "", wxFrame *parent = NULL,
                        int x = -1, int y = -1, Bool centre = TRUE);

#define wxCHOICE_HEIGHT 150
#define wxCHOICE_WIDTH 200

char *wxGetSingleChoice(char *message, char *caption,
                        int n, char *choices[], wxFrame *parent = NULL,
                        int x = -1, int y = -1, Bool centre = TRUE,
                        int width = wxCHOICE_WIDTH, int height = wxCHOICE_HEIGHT);

// Same as above but gets position in list of strings, instead of string,
// or -1 if no selection
int wxGetSingleChoiceIndex(char *message, char *caption,
                           int n, char *choices[], wxFrame *parent = NULL,
                           int x = -1, int y = -1, Bool centre = TRUE,
                           int width = wxCHOICE_WIDTH, int height = wxCHOICE_HEIGHT);

// Return client data instead
char *wxGetSingleChoiceData(char *message, char *caption,
                            int n, char *choices[], char *client_data[],
                            wxFrame *parent = NULL, int x = -1, int y = -1,
                            Bool centre = TRUE,
                            int width = wxCHOICE_WIDTH, int height = wxCHOICE_HEIGHT);
                           
// type is an 'or' (|) of wxOK, wxCANCEL, wxYES_NO
// Returns wxYES/NO/OK/CANCEL
int wxbMessageBox(char *message, char *caption = "Message", int style = wxOK|wxCENTRE,
  wxFrame *parent = NULL, int x = -1, int y = -1);

#define wxOPEN 1
#define wxSAVE 2
#define wxOVERWRITE_PROMPT 4
#define wxHIDE_READONLY 8

// File selector
char *wxFileSelector(char *message = "Select a file", char *default_path = NULL,
                     char *default_filename = NULL, char *default_extension = NULL,
                     char *wildcard = "*.*", int flags = 0,
                     wxFrame *parent = NULL, int x = -1, int y = -1);

#endif // IN_CPROTO
#endif // wxb_dialgh
